--[[
Name: RainbowEnchants
Description: Pick custom colors for enchants in item tooltips
Info: Used EnchantColors (by Smurfy) and RatingBuster (by Whitetooth) for coding ideas
Developed by: Necrathex (EU_Azjol-Nerub)
]]

-----------------
-- Addon Setup --
-----------------

-- Version
local MAJOR_VERSION = "0.1"
-- Libraries
local Compost = AceLibrary("Compost-2.0")
local L = AceLibrary("AceLocale-2.2"):new("RainbowEnchants")
-- AceAddon Initialization
RainbowEnchants = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")
RainbowEnchants.title = "RainbowEnchants"
RainbowEnchants.version = MAJOR_VERSION

-- Debugging
RainbowEnchants:SetDebugging(false)

---------------------
-- Saved Variables --
---------------------
-- Register DB
RainbowEnchants:RegisterDB("RainbowEnchantsDB")
-- Defaults
RainbowEnchants.DBDefaults = {
	permColor = '00ff00',
	tempColor = '00f0f0',
}
-- Register Defaults
RainbowEnchants:RegisterDefaults("profile", RainbowEnchants.DBDefaults)

---------------------------
-- Slash Command Options --
---------------------------

local consoleOptions = {
	type='group',
	args = {
		color_perm = {
    			type = 'color',
    			name = 'perm. enchant color',
    			desc = 'Edit permanent enchants color.',
    			get =	function() 
    				local hex = RainbowEnchants.db.profile.permColor
        			return tonumber(string.sub(hex,1,2),16)/255, tonumber(string.sub(hex,3,4),16)/255, tonumber(string.sub(hex,5),16)/255
        			end,
    			set =	function(r, g, b)
        			RainbowEnchants.db.profile.permColor = string.format("%2x%2x%2x", r*255, g*255, b*255)
    				end,
    		},
    		color_temp = {
    			type = 'color',
    			name = 'temp. enchant color',
    			desc = 'Edit temporary enchants color.',
    			get =	function() 
    				local hex = RainbowEnchants.db.profile.tempColor
        			return tonumber(string.sub(hex,1,2),16)/255, tonumber(string.sub(hex,3,4),16)/255, tonumber(string.sub(hex,5),16)/255
        			end,
    			set =	function(r, g, b)
        			RainbowEnchants.db.profile.tempColor = string.format("%2x%2x%2x", r*255, g*255, b*255)
    				end,
    		},
    		reset = {
			type = 'execute',
			name = 'reset to defaults',
			desc = 'Reset colors to defaults',
			func = function()
				RainbowEnchants.db.profile.permColor = RainbowEnchants.DBDefaults.permColor
				RainbowEnchants.db.profile.tempColor = RainbowEnchants.DBDefaults.tempColor
				RainbowEnchants:Print("Colors have been reset to defaults.")
			end,
    		},
	},
}

RainbowEnchants:RegisterChatCommand({"/re", "/RainbowEnchants"}, consoleOptions)

-------------------------------
-- Core Functions Start Here --
-------------------------------

function RainbowEnchants:OnInitialize()
	-- Hook tooltip SetX methods
	self:HookTooltipMethods()
end

function RainbowEnchants:OnEnable()
	-- Event to monitor shift, alt, ctrl keys
	self:RegisterEvent("MODIFIER_STATE_CHANGED")
end

function RainbowEnchants:OnDisable()
end

function RainbowEnchants:MODIFIER_STATE_CHANGED()
end

-------------------------------
-- Hook Tooltip SetX Methods --
-------------------------------
RainbowEnchants.TooltipList = {
	ItemRefTooltip,
	GameTooltip,
	ShoppingTooltip1,
	ShoppingTooltip2,
	-- EquipCompare support
	ComparisonTooltip1,
	ComparisonTooltip2,
	-- socketing interface support
	--ItemSocketingDescription, -- TODO
}

RainbowEnchants.MethodList = {
	"SetHyperlink",
	"SetBagItem",
	"SetInventoryItem",
	-- auction
	"SetAuctionItem",
	"SetAuctionSellItem",
	-- loot
	"SetLootItem",
	"SetLootRollItem",
	-- crafting
	"SetCraftItem",
	"SetTradeSkillItem",
	"SetTrainerService",-- profession trainer
	-- mail
	"SetInboxItem",
	"SetSendMailItem",
	-- buffs
	"SetPlayerBuff",
	-- quest log
	"SetQuestItem",
	"SetQuestLogItem",
	-- trade
	"SetTradePlayerItem",
	"SetTradeTargetItem",
	-- vendor tooltip
	"SetMerchantItem",
	"SetMerchantCompareItem",
	"SetBuybackItem",
	-- socketing interface support
	--"SetSocketedItem", -- main display
	"SetSocketGem", -- tooltips
	"SetExistingSocketGem", -- tooltips
}

function RainbowEnchants:HookTooltipMethods()
	for _, tooltip in ipairs(self.TooltipList) do
		if tooltip then
			for _, methodName in ipairs(self.MethodList) do
				hooksecurefunc(tooltip, methodName, self.ProcessTooltip)
			end
		end
	end
end

function RainbowEnchants.ProcessTooltip(tooltip, type, index)
	-- Active check
	if not RainbowEnchants:IsActive() then return end

	local firstenchant = true;
	for i = 2, tooltip:NumLines() do
		local fontString = getglobal(tooltip:GetName().."TextLeft"..i)
		local text = fontString:GetText()
		
		-- Look for white colored text (differently colored texts will be ignored, maybe change if in the future enchants will be green again?)
		if (string.sub(text,1,10) == '|cffffffff') then

			local enchantcolor = RainbowEnchants.db.profile.permColor; -- Default color is the permanent enchant color
			local allowrecolor = false;
			
			-- Fetch item link to check for enchants/suffix
			local itemlink = nil

			if (tooltip:GetName() == "ItemRefTooltip") then
				-- Chat link
				itemlink = type
			elseif (index ~= nil) then
				-- Tooltip
				local tmplink = nil
				if (type == "player" or type == "target") then
					-- Player frame, inspect frame
					tmplink = GetInventoryItemLink(type, index)
				elseif (type == "list" or type == "bidder" or type == "owner") then
					-- Auction frames
					tmplink = GetAuctionItemLink(type, index)
				else
					-- Inventory
					tmplink = GetContainerItemLink(type, index)
				end
				
				-- Convert extracted link to chat link format
				if (tmplink ~= nil) then
					for item in string.gmatch(tmplink, "|c%x+|Hitem:(%d+:%d+:%d+:%d+:%d+:%d+:%d+:%d+)|h%[.-%]|h|r") do
						if( item and item ~= "" ) then
							itemlink = "item:"..string.gsub(item, "^(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)$", "%1:%2:%3:%4:%5:%6:%7:%8");
						end
					end
				end
			else
			-- Can't get itemlink for this YET (wait for 'tooltip:GetItem()')
				return
			end
			
			-- Check suffix ('of the blahblah' items with enchant) and temporary enchants
			if (itemlink) then
				local found, _, enchant, suffix = string.find(itemlink, "^item:%d+:(%d+):%d+:%d+:%d+:%d+:(%d+):%d+")
				-- Permanent enchant is always first
				if (enchant ~= "0" and firstenchant) then
					allowrecolor = true;
					firstenchant = false;
				else -- Check for temporary enchant (they don't start with a +)
					if (string.sub(text,11,11) ~= '+') then
						allowrecolor = true;
						enchantcolor = RainbowEnchants.db.profile.tempColor;
					end
				end
			end
			
			if (allowrecolor) then
				text = '|cff' .. enchantcolor .. string.sub(text,11,string.len(text));
				fontString:SetText(text)
			end
		end
	end
end
